"use strict";
/*
 * Copyright 2018 The NATS Authors
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
var crc16_1 = require("./crc16");
var b32enc = require("base32-encode");
var b32dec = require("base32-decode");
var ed25519 = require("tweetnacl");
var nkeys_1 = require("./nkeys");
var util = require("./util");
var Codec = /** @class */ (function () {
    function Codec() {
    }
    Codec.encode = function (prefix, src) {
        return new Promise(function (resolve, reject) {
            if (!Buffer.isBuffer(src)) {
                reject(new nkeys_1.NKeysError(nkeys_1.NKeysErrorCode.SerializationError));
                return;
            }
            if (!nkeys_1.Prefixes.isValidPrefix(prefix)) {
                reject(new nkeys_1.NKeysError(nkeys_1.NKeysErrorCode.InvalidPrefixByte));
                return;
            }
            // offsets
            var payloadOffset = 1;
            var payloadLen = src.byteLength;
            var checkLen = 2;
            var cap = payloadOffset + payloadLen + checkLen;
            var checkOffset = payloadOffset + payloadLen;
            var raw = new Buffer(cap);
            raw[0] = prefix;
            src.copy(raw, payloadOffset);
            //calculate the checksum write it LE
            var checksum = crc16_1.crc16.checksum(raw.slice(0, checkOffset));
            raw.writeUInt16LE(checksum, checkOffset);
            // generate a base32 string - remove the padding
            var str = b32enc(Codec.toArrayBuffer(raw), 'RFC3548');
            str = str.replace(/=+$/, '');
            resolve(str);
        });
    };
    Codec.decode = function (src) {
        return new Promise(function (resolve, reject) {
            if (!nkeys_1.Prefixes.startsWithValidPrefix(src)) {
                reject(new nkeys_1.NKeysError(nkeys_1.NKeysErrorCode.InvalidPrefixByte));
                return;
            }
            var buf;
            try {
                buf = b32dec(src, 'RFC3548');
            }
            catch (ex) {
                reject(new nkeys_1.NKeysError(nkeys_1.NKeysErrorCode.InvalidEncoding, ex));
                return;
            }
            var raw = Buffer.from(buf);
            if (raw.byteLength < 3) {
                reject(new nkeys_1.NKeysError(nkeys_1.NKeysErrorCode.InvalidEncoding));
                return;
            }
            var checkOffset = raw.byteLength - 2;
            var checksum = raw.readUInt16LE(checkOffset);
            var payload = raw.slice(0, checkOffset);
            if (!crc16_1.crc16.validate(payload, checksum)) {
                reject(new nkeys_1.NKeysError(nkeys_1.NKeysErrorCode.InvalidChecksum));
                return;
            }
            resolve(payload);
        });
    };
    Codec.decodeExpectingPrefix = function (expected, src) {
        return new Promise(function (resolve, reject) {
            if (!nkeys_1.Prefixes.isValidPrefix(expected)) {
                reject(new nkeys_1.NKeysError(nkeys_1.NKeysErrorCode.InvalidPrefixByte));
                return;
            }
            Codec.decode(src)
                .then(function (buf) {
                if (buf[0] != expected) {
                    reject(new nkeys_1.NKeysError(nkeys_1.NKeysErrorCode.InvalidPrefixByte));
                    return;
                }
                resolve(buf.slice(1));
            }).catch(function (err) {
                reject(err);
            });
        });
    };
    Codec.decodeSeed = function (src) {
        return new Promise(function (resolve, reject) {
            Codec.decode(src)
                .then(function (raw) {
                if (raw.byteLength < 4) {
                    reject(new nkeys_1.NKeysError(nkeys_1.NKeysErrorCode.InvalidEncoding));
                    return;
                }
                var prefix = Codec.decodePrefix(raw);
                if (prefix[0] != nkeys_1.Prefix.Seed) {
                    reject(new nkeys_1.NKeysError(nkeys_1.NKeysErrorCode.InvalidSeed));
                    return;
                }
                if (!nkeys_1.Prefixes.isValidPublicPrefix(prefix[1])) {
                    reject(new nkeys_1.NKeysError(nkeys_1.NKeysErrorCode.InvalidSeed));
                    return;
                }
                resolve({ buf: raw.slice(2), prefix: prefix[1] });
            }).catch(function (err) {
                reject(err);
            });
        });
    };
    Codec.encodeSeed = function (role, src) {
        return new Promise(function (resolve, reject) {
            if (!Buffer.isBuffer(src)) {
                reject(new nkeys_1.NKeysError(nkeys_1.NKeysErrorCode.SerializationError));
                return;
            }
            if (!nkeys_1.Prefixes.isValidPublicPrefix(role)) {
                reject(new nkeys_1.NKeysError(nkeys_1.NKeysErrorCode.InvalidPrefixByte));
                return;
            }
            if (src.byteLength != ed25519.sign.secretKeyLength) {
                reject(new nkeys_1.NKeysError(nkeys_1.NKeysErrorCode.InvalidSeedLen));
                return;
            }
            // offsets for this token
            var payloadOffset = 2;
            var payloadLen = src.byteLength;
            var checkLen = 2;
            var cap = payloadOffset + payloadLen + checkLen;
            var checkOffset = payloadOffset + payloadLen;
            // make the prefixes human readable when encoded
            var prefix = Codec.encodePrefix(nkeys_1.Prefix.Seed, role);
            var raw = new Buffer(cap);
            prefix.copy(raw, 0, 0);
            src.copy(raw, payloadOffset, 0);
            //calculate the checksum write it LE
            var checksum = crc16_1.crc16.checksum(raw.slice(0, checkOffset));
            raw.writeUInt16LE(checksum, checkOffset);
            // generate a string
            // generate a base32 string - remove the padding
            var str = b32enc(Codec.toArrayBuffer(raw), 'RFC3548');
            str = str.replace(/=+$/, '');
            resolve(str);
        });
    };
    Codec.encodePrefix = function (kind, role) {
        // In order to make this human printable for both bytes, we need to do a little
        // bit manipulation to setup for base32 encoding which takes 5 bits at a time.
        var b1 = kind | (role >> 5);
        var b2 = (role & 31) << 3; // 31 = 00011111
        return new Buffer([b1, b2]);
    };
    Codec.decodePrefix = function (raw) {
        // Need to do the reverse from the printable representation to
        // get back to internal representation.
        var b1 = raw[0] & 248; // 248 = 11111000
        var b2 = (raw[0] & 7) << 5 | ((raw[1] & 248) >> 3); // 7 = 00000111
        var a = new Uint8Array(2);
        a[0] = b1;
        a[1] = b2;
        return a;
    };
    Codec.toArrayBuffer = util.toArrayBuffer();
    return Codec;
}());
exports.Codec = Codec;
//# sourceMappingURL=codec.js.map